require "cromwell_api"

module CromwellApi
module Rack
  class RequestTime

    DEFAULT_METRIC_NAME = "request_time".freeze

    attr_reader :config
    def initialize(app, config)
      @app = app
      @config = config
    end

    def call(env)
      start_time = Time.now
      response = @app.call(env)
      request_time = ((Time.now - start_time) * 1000).round

      if response[0].between? 200, 299
        client.send_metric metric_key(env), request_time
      end

      response
    end

    def client
      Thread.current[:cromwell_client] ||= begin
        options = {}
        non_option_keys = [:product, :host]
        config.each do |key, value|
          key = key.to_sym
          unless non_option_keys.include? key
            options[key] = value
          end
        end
        ::CromwellApi::Client.new config[:product], environment, config[:host], options
      end
    end

    def environment
      config.fetch :environment, (ENV.fetch "RACK_ENV", "production")
    end

    def metric_key(env)
      # Requires sinatra 1.4+
      if env["sinatra.route"]
        env["sinatra.route"]
      elsif (defined? ::Grape::Env::RACK_ROUTING_ARGS) || (defined? ::Grape::Env::GRAPE_ROUTING_ARGS)
        begin
          grape_route env
        rescue
          DEFAULT_METRIC_NAME
        end
      else
        DEFAULT_METRIC_NAME
      end
    end

    def grape_route(env)
      routing_args = if defined? ::Grape::Env::GRAPE_ROUTING_ARGS
                       env[::Grape::Env::GRAPE_ROUTING_ARGS]
                     else
                       env[::Grape::Env::RACK_ROUTING_ARGS]
                     end
      route_parts = env["PATH_INFO"].split "/"
      new_parts = route_parts.map do |part|
        next if part == ""
        route_arg = routing_args.find {|key, value| value == part}
        if route_arg
          ":#{route_arg[0]}"
        else
          part
        end
      end
      "#{env["REQUEST_METHOD"]} #{new_parts.join "/"}"
    end

  end
end
end
